@@ -434,6 +434,47 @@ class FT_Outline(Structure):
         ('flags',      c_int),
     ]
 
+# -----------------------------------------------------------------------------
+# Callback functions used in FT_Outline_Funcs
+
+FT_Outline_MoveToFunc  = CFUNCTYPE(c_int, POINTER(FT_Vector), py_object)
+FT_Outline_LineToFunc  = CFUNCTYPE(c_int, POINTER(FT_Vector), py_object)
+FT_Outline_ConicToFunc = CFUNCTYPE(c_int, POINTER(FT_Vector), POINTER(FT_Vector), py_object)
+FT_Outline_CubicToFunc = CFUNCTYPE(c_int, POINTER(FT_Vector), POINTER(FT_Vector), POINTER(FT_Vector), py_object)
+
+# -----------------------------------------------------------------------------
+# Struct of callback functions for FT_Outline_Decompose()
+
+class FT_Outline_Funcs(Structure):
+    '''
+    This structure holds a set of callbacks which are called by
+    FT_Outline_Decompose.
+
+    move_to: Callback when outline needs to jump to a new path component.
+
+    line_to: Callback to draw a straight line from the current position to
+             the control point.
+
+    conic_to: Callback to draw a second-order Bézier curve from the current
+              position using the passed control points.
+
+    curve_to: Callback to draw a third-order Bézier curve from the current
+              position using the passed control points.
+
+    shift: Passed to FreeType which will transform vectors via
+           `x = (x << shift) - delta` and `y = (y << shift) - delta`
+
+    delta: Passed to FreeType which will transform vectors via
+           `x = (x << shift) - delta` and `y = (y << shift) - delta`
+    '''
+    _fields_ = [
+        ('move_to', FT_Outline_MoveToFunc),
+        ('line_to', FT_Outline_LineToFunc),
+        ('conic_to', FT_Outline_ConicToFunc),
+        ('cubic_to', FT_Outline_CubicToFunc),
+        ('shift', c_int),
+        ('delta', FT_Pos),
+    ]
 
 # -----------------------------------------------------------------------------
 # The root glyph structure contains a given glyph image plus its advance width
@@ -942,3 +983,48 @@ class FT_BitmapGlyphRec(Structure):
         ('bitmap', FT_Bitmap)
     ]
 FT_BitmapGlyph = POINTER(FT_BitmapGlyphRec)
+
+
+# -----------------------------------------------------------------------------
+# Structures related to variable fonts as used in the various VF routines.
+#
+class FT_Var_Axis(Structure):
+    '''
+    A structure to model a given axis in design space for Multiple Masters,
+    TrueType GX, and OpenType variation fonts.
+    '''
+    _fields_ = [
+        ('name', FT_String_p),
+        ('minimum', FT_Fixed),
+        ('default', FT_Fixed),
+        ('maximum', FT_Fixed),
+        ('tag', FT_ULong),
+        ('strid', FT_UInt)
+    ]
+
+
+class FT_Var_Named_Style(Structure):
+    '''
+    A structure to model a named instance in a TrueType GX or OpenType
+    variation font.
+    '''
+    _fields_ = [
+        ('coords', POINTER(FT_Fixed)),
+        ('strid', FT_UInt),
+        ('psid', FT_UInt)
+    ]
+
+
+class FT_MM_Var(Structure):
+    '''
+    A structure to model the axes and space of an Adobe MM, TrueType GX,
+    or OpenType variation font.
+    Some fields are specific to one format and not to the others.
+    '''
+    _fields_ = [
+        ('num_axis', FT_UInt),
+        ('num_designs', FT_UInt),
+        ('num_namedstyles', FT_UInt),
+        ('axis', POINTER(FT_Var_Axis)),
+        ('namedstyle', POINTER(FT_Var_Named_Style))
+    ]
